"""InstrumentLab - controlling test instruments like power supplies and oscilloscopes."""

__version__ = "0.0.5"

from .base.config import Config

from .base.oscilloscope.oscillocope_constants import OscilloscopeConstants
